@@ -48,6 +48,7 @@ def cli(
     return_unknown_args: Literal[False] = False,
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> OutT: ...
@@ -64,6 +65,7 @@ def cli(
     return_unknown_args: Literal[True],
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> tuple[OutT, list[str]]: ...
@@ -83,6 +85,7 @@ def cli(
     return_unknown_args: Literal[False] = False,
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> OutT: ...
@@ -102,6 +105,7 @@ def cli(
     return_unknown_args: Literal[True],
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> tuple[OutT, list[str]]: ...
@@ -117,6 +121,7 @@ def cli(
     return_unknown_args: bool = False,
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
     **deprecated_kwargs,
@@ -185,6 +190,8 @@ def cli(
         console_outputs: If set to False, suppresses parsing errors and help messages.
             This is useful in distributed settings where tyro.cli() is called from multiple
             workers but console output is only desired from the main process.
+        add_help: Add a -h/--help option to the parser. This mirrors the argument from
+            :py:class:`argparse.ArgumentParser()`.
         config: A sequence of configuration marker objects from :mod:`tyro.conf`. This
             allows applying markers globally instead of annotating individual fields.
             For example: ``tyro.cli(Config, config=(tyro.conf.PositionalRequiredArgs,))``
@@ -213,6 +220,7 @@ def cli(
             return_unknown_args=return_unknown_args,
             use_underscores=use_underscores,
             console_outputs=console_outputs,
+            add_help=add_help,
             config=config,
             registry=registry,
             **deprecated_kwargs,
@@ -239,6 +247,7 @@ def get_parser(
     default: None | OutT = None,
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> argparse.ArgumentParser: ...
@@ -253,6 +262,7 @@ def get_parser(
     default: None | OutT = None,
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> argparse.ArgumentParser: ...
@@ -268,6 +278,7 @@ def get_parser(
     default: None | OutT = None,
     use_underscores: bool = False,
     console_outputs: bool = True,
+    add_help: bool = True,
     config: None | Sequence[conf._markers.Marker] = None,
     registry: None | ConstructorRegistry = None,
 ) -> argparse.ArgumentParser:
@@ -284,6 +295,8 @@ def get_parser(
         default: An instance to use for default values.
         use_underscores: If True, uses underscores as word delimiters in the help text.
         console_outputs: If set to False, suppresses parsing errors and help messages.
+        add_help: Add a -h/--help option to the parser. This mirrors the argument from
+            :py:class:`argparse.ArgumentParser()`.
         config: A sequence of configuration marker objects from :mod:`tyro.conf`.
         registry: A :class:`tyro.constructors.ConstructorRegistry` instance containing custom
             constructor rules.
@@ -301,6 +314,7 @@ def get_parser(
                 return_unknown_args=False,
                 use_underscores=use_underscores,
                 console_outputs=console_outputs,
+                add_help=add_help,
                 config=config,
                 registry=registry,
             ),
@@ -317,6 +331,7 @@ def _cli_impl(
     return_parser: bool,
     return_unknown_args: bool,
     console_outputs: bool,
+    add_help: bool,
     config: None | Sequence[conf._markers.Marker],
     registry: None | ConstructorRegistry = None,
     **deprecated_kwargs,
@@ -359,6 +374,10 @@ def _cli_impl(
     # We wrap our type with a dummy dataclass if it can't be treated as a nested type.
     # For example: passing in f=int will result in a dataclass with a single field
     # typed as int.
+    #
+    # Why don't we always use a dummy dataclass?
+    # => Docstrings for inner structs are currently lost when we nest struct types.
+    f = _resolver.TypeParamResolver.resolve_params_and_aliases(f)
     if not _fields.is_struct_type(cast(type, f), default_instance_internal):
         dummy_field = cast(
             dataclasses.Field,
@@ -372,7 +391,6 @@ def _cli_impl(
         default_instance_internal = f(default_instance_internal)  # type: ignore
         dummy_wrapped = True
     else:
-        f = _resolver.TypeParamResolver.resolve_params_and_aliases(f)
         dummy_wrapped = False
 
     # Read and fix arguments. If the user passes in --field_name instead of
@@ -446,6 +464,7 @@ def _cli_impl(
                 default_instance=default_instance_internal,  # Overrides for default values.
                 intern_prefix="",  # Used for recursive calls.
                 extern_prefix="",  # Used for recursive calls.
+                add_help=add_help,
             )
     else:
         parser_spec = _parsers.ParserSpecification.from_callable_or_type(
@@ -456,6 +475,7 @@ def _cli_impl(
             default_instance=default_instance_internal,  # Overrides for default values.
             intern_prefix="",  # Used for recursive calls.
             extern_prefix="",  # Used for recursive calls.
+            add_help=add_help,
         )
 
     # Generate parser!
@@ -464,6 +484,7 @@ def _cli_impl(
             prog=prog,
             formatter_class=_argparse_formatter.TyroArgparseHelpFormatter,
             allow_abbrev=False,
+            add_help=add_help,
         )
         parser._parser_specification = parser_spec
         parser._parsing_known_args = return_unknown_args
@@ -571,8 +592,14 @@ def _cli_impl(
                                         ),
                                         pad=(0, 0, 0, 4),
                                     ),
-                                    Rule(style=Style(color="red")),
-                                    f"For full helptext, see [bold]{parser.prog} --help[/bold]",
+                                    *(
+                                        [
+                                            Rule(style=Style(color="red")),
+                                            f"For full helptext, see [bold]{parser.prog} --help[/bold]",
+                                        ]
+                                        if parser.add_help
+                                        else []
+                                    ),
                                 ]
                             ),
                         ),
