@@ -7,6 +7,7 @@ import numbers
 import warnings
 from typing import Any, Callable, Dict, List, Set, Tuple, Type, TypeVar, Union, cast
 
+from rich.text import Text
 from typing_extensions import Annotated, get_args, get_origin
 
 from tyro.constructors._registry import ConstructorRegistry
@@ -24,7 +25,9 @@ from . import (
     _subcommand_matching,
 )
 from ._typing import TypeForm
+from ._typing_compat import is_typing_union
 from .conf import _confstruct, _markers
+from .conf._mutex_group import _MutexGroupConfig
 from .constructors._primitive_spec import (
     PrimitiveConstructorSpec,
     UnsupportedTypeAnnotationError,
@@ -56,6 +59,7 @@ class ParserSpecification:
     extern_prefix: str
     has_required_args: bool
     consolidate_subcommand_args: bool
+    add_help: bool
 
     @staticmethod
     def from_callable_or_type(
@@ -68,6 +72,7 @@ class ParserSpecification:
         ],
         intern_prefix: str,
         extern_prefix: str,
+        add_help: bool,
         subcommand_prefix: str = "",
         support_single_arg_types: bool = False,
     ) -> ParserSpecification:
@@ -105,7 +110,7 @@ class ParserSpecification:
                 default_instance=default_instance,
                 support_single_arg_types=support_single_arg_types,
             )
-            assert not isinstance(out, UnsupportedStructTypeMessage)
+            assert not isinstance(out, UnsupportedStructTypeMessage), out
             f, field_list = out
 
         has_required_args = False
@@ -124,6 +129,7 @@ class ParserSpecification:
                 intern_prefix=intern_prefix,
                 extern_prefix=extern_prefix,
                 subcommand_prefix=subcommand_prefix,
+                add_help=add_help,
             )
             if isinstance(field_out, _arguments.ArgumentDefinition):
                 # Handle single arguments.
@@ -197,6 +203,7 @@ class ParserSpecification:
             extern_prefix=extern_prefix,
             has_required_args=has_required_args,
             consolidate_subcommand_args=consolidate_subcommand_args,
+            add_help=add_help,
         )
 
     def apply(
@@ -248,6 +255,10 @@ class ParserSpecification:
         self,
         parser: argparse.ArgumentParser,
         parent: ParserSpecification | None = None,
+        exclusive_group_from_group_conf: Dict[
+            _MutexGroupConfig, argparse._MutuallyExclusiveGroup
+        ]
+        | None = None,
     ) -> None:
         """Create defined arguments and subparsers."""
 
@@ -265,49 +276,69 @@ class ParserSpecification:
         positional_group = parser._action_groups[0]
         assert positional_group.title == "positional arguments"
 
+        # Inherit mutex groups from parent or create new dict
+        if exclusive_group_from_group_conf is None:
+            exclusive_group_from_group_conf = {}
+
         # Add each argument group. Groups with only suppressed arguments won't
         # be added.
         for arg in self.args:
             # Don't add suppressed arguments to the parser.
             if arg.is_suppressed():
                 continue
-
-            group_name = (
-                arg.extern_prefix
-                if arg.field.argconf.name != ""
-                # If the field name is "erased", we'll place the argument in
-                # the parent's group.
-                #
-                # This is to avoid "issue 1" in:
-                # https://github.com/brentyi/tyro/issues/183
-                #
-                # Setting `tyro.conf.arg(name="")` should generally be
-                # discouraged, so this will rarely matter.
-                else arg.extern_prefix.rpartition(".")[0]
-            )
-            if group_name not in group_from_group_name:
-                description = (
-                    parent.helptext_from_intern_prefixed_field_name.get(
-                        arg.intern_prefix
-                    )
-                    if parent is not None
-                    else None
-                )
-                group_from_group_name[group_name] = parser.add_argument_group(
-                    format_group_name(group_name),
-                    description=description,
-                )
-
-            # Add each argument.
-            if arg.field.is_positional():
+            elif arg.field.is_positional():
                 arg.add_argument(positional_group)
                 continue
-
-            assert group_name in group_from_group_name
-            arg.add_argument(group_from_group_name[group_name])
+            elif arg.field.mutex_group is not None:
+                group_conf = arg.field.mutex_group
+                if group_conf not in exclusive_group_from_group_conf:
+                    exclusive_group_from_group_conf[group_conf] = (
+                        parser.add_argument_group(
+                            "mutually exclusive",
+                            description=_argparse_formatter.str_from_rich(
+                                Text.from_markup(
+                                    "Exactly one argument must be passed in. [bright_red](required)[/bright_red]"
+                                )
+                            )
+                            if group_conf.required
+                            else "At most one argument can be overridden.",
+                        ).add_mutually_exclusive_group(required=group_conf.required)
+                    )
+                arg.add_argument(exclusive_group_from_group_conf[group_conf])
+            else:
+                group_name = (
+                    arg.extern_prefix
+                    if arg.field.argconf.name != ""
+                    # If the field name is "erased", we'll place the argument in
+                    # the parent's group.
+                    #
+                    # This is to avoid "issue 1" in:
+                    # https://github.com/brentyi/tyro/issues/183
+                    #
+                    # Setting `tyro.conf.arg(name="")` should generally be
+                    # discouraged, so this will rarely matter.
+                    else arg.extern_prefix.rpartition(".")[0]
+                )
+                if group_name not in group_from_group_name:
+                    description = (
+                        parent.helptext_from_intern_prefixed_field_name.get(
+                            arg.intern_prefix
+                        )
+                        if parent is not None
+                        else None
+                    )
+                    group_from_group_name[group_name] = parser.add_argument_group(
+                        format_group_name(group_name),
+                        description=description,
+                    )
+                arg.add_argument(group_from_group_name[group_name])
 
         for child in self.child_from_prefix.values():
-            child.apply_args(parser, parent=self)
+            child.apply_args(
+                parser,
+                parent=self,
+                exclusive_group_from_group_conf=exclusive_group_from_group_conf,
+            )
 
 
 def handle_field(
@@ -316,6 +347,7 @@ def handle_field(
     intern_prefix: str,
     extern_prefix: str,
     subcommand_prefix: str,
+    add_help: bool,
 ) -> Union[
     _arguments.ArgumentDefinition,
     ParserSpecification,
@@ -369,15 +401,10 @@ def handle_field(
                 extern_prefix=_strings.make_field_name(
                     [extern_prefix, field.extern_name]
                 ),
+                add_help=add_help,
             )
             if subparsers_attempt is not None:
-                if subparsers_attempt.default_parser is not None and (
-                    _markers.AvoidSubcommands in field.markers
-                ):
-                    # Don't make a subparser, just use the default subcommand.
-                    return subparsers_attempt.default_parser
-                else:
-                    return subparsers_attempt
+                return subparsers_attempt
 
         # (2) Handle nested callables.
         if _fields.is_struct_type(field.type, field.default):
@@ -395,6 +422,7 @@ def handle_field(
                     if field.argconf.prefix_name in (True, None)
                     else field.extern_name
                 ),
+                add_help=add_help,
                 subcommand_prefix=subcommand_prefix,
                 support_single_arg_types=False,
             )
@@ -428,22 +456,20 @@ class SubparsersSpecification:
         parent_classes: Set[Type[Any]],
         intern_prefix: str,
         extern_prefix: str,
-    ) -> SubparsersSpecification | None:
+        add_help: bool,
+    ) -> SubparsersSpecification | ParserSpecification | None:
+        """From a field: return either a subparser specification, a parser
+        specification for subcommands when `tyro.conf.AvoidSubcommands` is used
+        and a default is set, or `None` if the field does not create a
+        subparser."""
         # Union of classes should create subparsers.
         typ = _resolver.unwrap_annotated(field.type_stripped)
-        if get_origin(typ) not in (Union, _resolver.UnionType):
+        if not is_typing_union(get_origin(typ)):
             return None
 
         # We don't use sets here to retain order of subcommands.
         options: List[Union[type, Callable]]
         options = [typ for typ in get_args(typ)]
-        options = [
-            (
-                # Cast seems unnecessary but needed in mypy... (1.4.1)
-                cast(Callable, none_proxy) if o is type(None) else o
-            )
-            for o in options
-        ]
 
         # If specified, swap types using tyro.conf.subcommand(constructor=...).
         for i, option in enumerate(options):
@@ -464,8 +490,7 @@ class SubparsersSpecification:
         # Exit if we don't contain any nested types.
         if not any(
             [
-                o is not none_proxy
-                and _fields.is_struct_type(cast(type, o), _singleton.MISSING_NONPROP)
+                _fields.is_struct_type(cast(type, o), _singleton.MISSING_NONPROP)
                 for o in options
             ]
         ):
@@ -484,20 +509,17 @@ class SubparsersSpecification:
                     if _markers.OmitSubcommandPrefixes in field.markers
                     else extern_prefix
                 ),
-                type(None) if option_unwrapped is none_proxy else cast(type, option),
+                cast(type, option),
             )
             if subcommand_name in subcommand_type_from_name:
                 # Raise a warning that the subcommand already exists
                 original_type = subcommand_type_from_name[subcommand_name]
-                new_type = (
-                    type(None) if option_unwrapped is none_proxy else option_unwrapped
-                )
                 original_type_full_name = (
                     f"{original_type.__module__}.{original_type.__name__}"
                 )
                 new_type_full_name = (
-                    f"{new_type.__module__}.{new_type.__name__}"
-                    if new_type is not None
+                    f"{option_unwrapped.__module__}.{option_unwrapped.__name__}"
+                    if option_unwrapped is not None
                     else "None"
                 )
 
@@ -522,23 +544,11 @@ class SubparsersSpecification:
         # If a field default is provided, try to find a matching subcommand name.
         default_name = None
         if field.default not in _singleton.MISSING_AND_MISSING_NONPROP:
-            # Subcommand matcher won't work with `none_proxy`.
-            if field.default is None:
-                default_name = next(
-                    iter(
-                        filter(
-                            lambda pair: pair[1] is none_proxy,
-                            subcommand_type_from_name.items(),
-                        )
-                    )
-                )[0]
-            else:
-                default_name = _subcommand_matching.match_subcommand(
-                    field.default,
-                    subcommand_config_from_name,
-                    subcommand_type_from_name,
-                )
-
+            default_name = _subcommand_matching.match_subcommand(
+                field.default,
+                subcommand_config_from_name,
+                subcommand_type_from_name,
+            )
             assert default_name is not None, (
                 f"`{extern_prefix}` was provided a default value of type"
                 f" {type(field.default)} but no matching subcommand was found. A"
@@ -548,6 +558,21 @@ class SubparsersSpecification:
                 "is particularly limited for custom generic types."
             )
 
+        # Handle `tyro.conf.AvoidSubcommands` with a default value.
+        if default_name is not None and _markers.AvoidSubcommands in field.markers:
+            return ParserSpecification.from_callable_or_type(
+                subcommand_type_from_name[default_name],
+                markers=field.markers,
+                description=None,
+                parent_classes=parent_classes,
+                default_instance=field.default,
+                intern_prefix=intern_prefix,
+                extern_prefix=extern_prefix,
+                add_help=add_help,
+                subcommand_prefix=intern_prefix,
+                support_single_arg_types=True,
+            )
+
         # Add subcommands for each option.
         parser_from_name: Dict[str, ParserSpecification] = {}
         for option in options:
@@ -557,7 +582,7 @@ class SubparsersSpecification:
                     if _markers.OmitSubcommandPrefixes in field.markers
                     else extern_prefix
                 ),
-                type(None) if option is none_proxy else cast(type, option),
+                cast(type, option),
             )
 
             # Get a subcommand config: either pulled from the type annotations or the
@@ -606,6 +631,7 @@ class SubparsersSpecification:
                     default_instance=subcommand_config.default,
                     intern_prefix=intern_prefix,
                     extern_prefix=extern_prefix,
+                    add_help=add_help,
                     subcommand_prefix=intern_prefix,
                     support_single_arg_types=True,
                 )
@@ -714,6 +740,7 @@ class SubparsersSpecification:
                 formatter_class=_argparse_formatter.TyroArgparseHelpFormatter,
                 help=helptext,
                 allow_abbrev=False,
+                add_help=parent_parser.add_help,
             )
 
             # Attributes used for error message generation.
@@ -748,7 +775,3 @@ def add_subparsers_to_leaves(
         parser_from_name=new_parsers_from_name,
         required=root.required or leaf.required,
     )
-
-
-def none_proxy() -> None:
-    return None
