@@ -6,15 +6,13 @@ import collections.abc
 import copy
 import dataclasses
 import inspect
+import sys
 import types
-import typing
 import warnings
 from typing import (
     Any,
     Callable,
-    ClassVar,
     Dict,
-    Generic,
     List,
     Literal,
     Sequence,
@@ -32,6 +30,7 @@ from typing_extensions import (
     Annotated,
     Final,
     ForwardRef,
+    NoDefault,
     ReadOnly,
     Self,
     TypeAliasType,
@@ -44,11 +43,15 @@ from typing_extensions import (
 from . import _unsafe_cache, conf
 from ._singleton import MISSING_AND_MISSING_NONPROP
 from ._typing import TypeForm
-
-# typing_extensions.TypeAliasType and typing.TypeAliasType are not the same
-# object in typing_extensions 4.13.0! This can break an isinstance() check we
-# use below.
-TypeAliasTypeAlternate = getattr(typing, "TypeAliasType", TypeAliasType)
+from ._typing_compat import (
+    is_typing_annotated,
+    is_typing_classvar,
+    is_typing_generic,
+    is_typing_protocol,
+    is_typing_typealiastype,
+    is_typing_union,
+)
+from ._warnings import TyroWarning
 
 UnionType = getattr(types, "UnionType", Union)
 """Same as types.UnionType, but points to typing.Union for older versions of
@@ -102,7 +105,7 @@ def resolved_fields(cls: TypeForm) -> List[dataclasses.Field]:
         field.type = annotations[field.name]
 
         # Skip ClassVars.
-        if get_origin(field.type) is ClassVar:
+        if is_typing_classvar(get_origin(field.type)):
             continue
 
         # Unwrap InitVar types.
@@ -130,7 +133,7 @@ def resolve_newtype_and_aliases(
     typ: TypeOrCallableOrNone,
 ) -> TypeOrCallableOrNone:
     # Handle type aliases, eg via the `type` statement in Python 3.12.
-    if isinstance(typ, (TypeAliasType, TypeAliasTypeAlternate)):
+    if is_typing_typealiastype(type(typ)):
         typ_cast = cast(TypeAliasType, typ)
         return Annotated[  # type: ignore
             (
@@ -185,11 +188,11 @@ def narrow_subtypes(
         superclass = unwrap_annotated(typ)
 
         # For Python 3.10.
-        if get_origin(superclass) is Union:
+        if is_typing_union(get_origin(superclass)):
             return typ
 
         if superclass is Any or issubclass(potential_subclass, superclass):  # type: ignore
-            if get_origin(typ) is Annotated:
+            if is_typing_annotated(get_origin(typ)):
                 return Annotated[(potential_subclass,) + get_args(typ)[1:]]  # type: ignore
             typ = cast(TypeOrCallable, potential_subclass)
     except TypeError:
@@ -330,16 +333,14 @@ def unwrap_annotated(
         else:
             return get_args(typ)[0]
 
-    # Check for __tyro_markers__ from @configure.
-    if hasattr(typ, "__tyro_markers__"):
-        if search_type == "all":
-            targets = getattr(typ, "__tyro_markers__")
-        else:
-            targets = tuple(
-                x
-                for x in getattr(typ, "__tyro_markers__")
-                if isinstance(x, search_type)  # type: ignore
-            )
+    # Check for __tyro_markers__ from @configure. Use `__dict__` instead of
+    # getattr() to prevent inheritance.
+    if hasattr(typ, "__dict__") and "__tyro_markers__" in typ.__dict__:
+        targets = tuple(
+            x
+            for x in typ.__dict__["__tyro_markers__"]
+            if search_type == "all" or isinstance(x, search_type)  # type: ignore
+        )
     else:
         targets = ()
 
@@ -378,7 +379,9 @@ class TypeParamResolver:
 
     @staticmethod
     def resolve_params_and_aliases(
-        typ: TypeOrCallable, seen: set[Any] | None = None
+        typ: TypeOrCallable,
+        seen: set[Any] | None = None,
+        ignore_confstruct: bool = False,
     ) -> TypeOrCallable:
         """Apply type parameter assignments based on the current context."""
 
@@ -392,13 +395,20 @@ class TypeParamResolver:
             seen.add(typ)
 
         # Resolve types recursively.
-        return TypeParamResolver._resolve_type_params(typ, seen=seen)
+        return TypeParamResolver._resolve_type_params(
+            typ, seen=seen, ignore_confstruct=ignore_confstruct
+        )
 
     @staticmethod
-    def _resolve_type_params(typ: TypeOrCallable, seen: set[Any]) -> TypeOrCallable:
+    def _resolve_type_params(
+        typ: TypeOrCallable,
+        seen: set[Any],
+        ignore_confstruct: bool,
+    ) -> TypeOrCallable:
         """Implementation of resolve_type_params(), which doesn't consider cycles."""
         # Handle aliases.
-        typ = swap_type_using_confstruct(typ)
+        if not ignore_confstruct:
+            typ = swap_type_using_confstruct(typ)
         typ = resolve_newtype_and_aliases(typ)
         GenericAlias = getattr(types, "GenericAlias", None)
         if GenericAlias is not None and isinstance(typ, GenericAlias):
@@ -409,11 +419,13 @@ class TypeParamResolver:
                 type_from_typevar = {}
                 for k, v in zip(type_params, get_args(typ)):
                     type_from_typevar[k] = TypeParamResolver._resolve_type_params(
-                        v, seen=seen
+                        v, seen=seen, ignore_confstruct=ignore_confstruct
                     )
                 typ = typ.__value__  # type: ignore
                 with TypeParamAssignmentContext(typ, type_from_typevar):
-                    return TypeParamResolver._resolve_type_params(typ, seen=seen)
+                    return TypeParamResolver._resolve_type_params(
+                        typ, seen=seen, ignore_confstruct=ignore_confstruct
+                    )
 
         # Search for type parameter assignments.
         for type_from_typevar in reversed(TypeParamResolver.param_assignments):
@@ -422,11 +434,19 @@ class TypeParamResolver:
 
         # Found a TypeVar that isn't bound.
         if isinstance(cast(Any, typ), TypeVar):
+            # Check for TypeVar default (PEP 696, available via typing_extensions).
+            default = getattr(typ, "__default__", NoDefault)
+            # If __default__ exists and is not the NoDefault sentinel, use it.
+            if default is not NoDefault:
+                # We have a valid default, use it without warning.
+                return default  # type: ignore
+
             bound = getattr(typ, "__bound__", None)
             if bound is not None:
                 # Try to infer type from TypeVar bound.
                 warnings.warn(
-                    f"Could not resolve type parameter {typ}. Type parameter resolution is not always possible in @staticmethod or @classmethod."
+                    f"Could not resolve type parameter {typ}. Type parameter resolution is not always possible in @staticmethod or @classmethod.",
+                    category=TyroWarning,
                 )
                 return bound
 
@@ -434,12 +454,14 @@ class TypeParamResolver:
             if len(constraints) > 0:
                 # Try to infer type from TypeVar constraints.
                 warnings.warn(
-                    f"Could not resolve type parameter {typ}. Type parameter resolution is not always possible in @staticmethod or @classmethod."
+                    f"Could not resolve type parameter {typ}. Type parameter resolution is not always possible in @staticmethod or @classmethod.",
+                    category=TyroWarning,
                 )
                 return Union[constraints]  # type: ignore
 
             warnings.warn(
-                f"Could not resolve type parameter {typ}. Type parameter resolution is not always possible in @staticmethod or @classmethod."
+                f"Could not resolve type parameter {typ}. Type parameter resolution is not always possible in @staticmethod or @classmethod.",
+                category=TyroWarning,
             )
             return Any  # type: ignore
 
@@ -447,7 +469,7 @@ class TypeParamResolver:
         args = get_args(typ)
         callable_was_flattened = False
         if len(args) > 0:
-            if origin is Annotated:
+            if is_typing_annotated(origin):
                 args = args[:1]
             if origin is collections.abc.Callable and isinstance(args[0], list):
                 args = tuple(args[0]) + args[1:]
@@ -525,7 +547,7 @@ def expand_union_types(typ: TypeOrCallable, default_instance: Any) -> TypeOrCall
     In this case, we raise a warning, then add the type of the default value to the
     union. Loosely motivated by: https://github.com/brentyi/tyro/issues/20
     """
-    if get_origin(typ) is not Union:
+    if not is_typing_union(get_origin(typ)):
         return typ
 
     options = get_args(typ)
@@ -538,7 +560,8 @@ def expand_union_types(typ: TypeOrCallable, default_instance: Any) -> TypeOrCall
         ):
             warnings.warn(
                 f"{type(default_instance)} does not match any type in Union:"
-                f" {options_unwrapped}"
+                f" {options_unwrapped}",
+                category=TyroWarning,
             )
             return Union[options + (type(default_instance),)]  # type: ignore
     except TypeError:
@@ -591,7 +614,7 @@ def resolve_generic_types(
     class, and a mapping from typevars to concrete types."""
 
     annotations: Tuple[Any, ...] = ()
-    if get_origin(typ) is Annotated:
+    if is_typing_annotated(get_origin(typ)):
         # ^We need this `if` statement for an obscure edge case: when `cls` is a
         # function with `__tyro_markers__` set, we don't want/need to return
         # Annotated[func, markers].
@@ -723,7 +746,12 @@ def get_type_hints_resolve_type_params(
     context_from_origin_type: dict[Any, TypeParamAssignmentContext] = {}
 
     def recurse_superclass_context(obj: Any) -> None:
-        if get_origin(obj) is Generic or obj is object:
+        origin_cls = get_origin(obj)
+        if (
+            is_typing_generic(origin_cls)
+            or is_typing_protocol(origin_cls)
+            or obj is object
+        ):
             return
         context = TypeParamResolver.get_assignment_context(obj)
         if context.origin_type in context_from_origin_type:
@@ -753,9 +781,18 @@ def get_type_hints_resolve_type_params(
         # Substitution for forwarded type parameters; if we have:
         #     class A[T](Base[T]): ...
         # and we're resolving A[int], the base class should be treated as Base[int].
+        #
+        # We set `ignore_confstruct=True` to avoid swapping types from
+        # `tyro.conf.arg` and `tyro.conf.subcommand`'s `constructor_factory`
+        # attributes, which might be applied using the `@tyro.conf.configure`
+        # decorator. These attributes should be ignored when traversing
+        # inheritance hierarchies.
         with context:
             resolved_bases = [
-                TypeParamResolver.resolve_params_and_aliases(base) for base in bases
+                TypeParamResolver.resolve_params_and_aliases(
+                    base, ignore_confstruct=True
+                )
+                for base in bases
             ]
 
         # Recursively resolve type parameters for all bases.
@@ -774,7 +811,19 @@ def get_type_hints_resolve_type_params(
         raw_hints = _get_type_hints_backported_syntax(
             origin_type, include_extras=include_extras
         )
-        keys = set(origin_type.__dict__.get("__annotations__", {}).keys())
+
+        # Explicit version check avoids an edge case for inherited generics.
+        # Specifically: tests/test_nested.py::test_generic_inherited
+
+        # if "__annotations__" in origin_type.__dict__:
+        if sys.version_info < (3, 14):
+            # Python 3.8~3.13.
+            keys = set(origin_type.__dict__.get("__annotations__", {}).keys())
+        elif hasattr(origin_type, "__annotations__"):
+            # Python 3.14.
+            keys = set(getattr(origin_type, "__annotations__").keys())
+        else:
+            keys = set()
 
         # Pydantic generics need special handling.
         pydantic_generic_metadata = getattr(
@@ -818,6 +867,8 @@ def _get_type_hints_backported_syntax(
                 globalns = getattr(obj, "__globals__", None)
 
                 # Get global namespace for classes.
+                if globalns is None and hasattr(obj, "__module__"):
+                    globalns = sys.modules[getattr(obj, "__module__")].__dict__
                 if globalns is None and hasattr(globalns, "__init__"):
                     globalns = getattr(getattr(obj, "__init__"), "__globals__", None)
 
