@@ -1,3 +1,4 @@
+# mypy: ignore-errors
 import argparse
 import contextlib
 import dataclasses
@@ -87,6 +88,39 @@ def test_omit_subcommand_prefix() -> None:
     )
 
 
+def test_avoid_subparser_with_unsupported_union_member() -> None:
+    @dataclasses.dataclass
+    class Dummy:
+        pass
+
+    @dataclasses.dataclass
+    class Container:
+        x: Dict[str, Dummy]
+        y: str
+
+    @dataclasses.dataclass
+    class Config:
+        a: int = 3
+        b: Union[Container, None] = None
+
+    assert tyro.cli(
+        Config,
+        config=(tyro.conf.AvoidSubcommands,),
+        args=["--a", "7"],
+    ) == Config(7, None)
+    assert tyro.cli(
+        Config,
+        config=(tyro.conf.AvoidSubcommands,),
+        args=["--a", "5"],
+    ) == Config(5, None)
+    with pytest.raises(SystemExit):
+        assert tyro.cli(
+            Config,
+            config=(tyro.conf.AvoidSubcommands,),
+            args=["--a", "5", "--b.y", "hello"],
+        )
+
+
 def test_avoid_subparser_with_default() -> None:
     @dataclasses.dataclass
     class DefaultInstanceHTTPServer:
@@ -1641,11 +1675,11 @@ def test_consolidate_subcommand_args_optional() -> None:
         sgd_foo: float = 1.0
 
     def _constructor() -> Type[OptimizerConfig]:
-        cfgs = [
+        cfgs = (
             Annotated[AdamConfig, tyro.conf.subcommand(name="adam")],
             Annotated[SGDConfig, tyro.conf.subcommand(name="sgd")],
-        ]
-        return Union.__getitem__(tuple(cfgs))  # type: ignore
+        )
+        return Union[cfgs]  # type: ignore
 
     # Required because of --x.
     @dataclasses.dataclass
@@ -1745,13 +1779,13 @@ def test_default_subcommand_consistency() -> None:
         sgd_foo: float = 1.0
 
     def _constructor() -> Any:
-        cfgs = [
+        cfgs = (
             Annotated[SGDConfig, tyro.conf.subcommand(name="sgd", default=SGDConfig())],
             Annotated[
                 AdamConfig, tyro.conf.subcommand(name="adam", default=AdamConfig())
             ],
-        ]
-        return Union.__getitem__(tuple(cfgs))  # type: ignore
+        )
+        return Union[cfgs]  # type: ignore
 
     CLIOptimizer = Annotated[
         OptimizerConfig,
@@ -1952,7 +1986,7 @@ class UnionCommand3:
     name: str = "default"
 
 
-def test_union_with_empty_config():
+def test_union_with_empty_config() -> None:
     """Union types should work with empty config tuple."""
     result = tyro.cli(
         Union[UnionCommand1, UnionCommand2],
@@ -1962,7 +1996,7 @@ def test_union_with_empty_config():
     assert result == UnionCommand1(arg1="test", flag1=False)
 
 
-def test_union_with_flag_create_pairs_off():
+def test_union_with_flag_create_pairs_off() -> None:
     """Union types should work with FlagCreatePairsOff config."""
     result = tyro.cli(
         Union[UnionCommand1, UnionCommand2],
@@ -1972,7 +2006,7 @@ def test_union_with_flag_create_pairs_off():
     assert result == UnionCommand1(arg1="test", flag1=True)
 
 
-def test_union_with_flag_conversion_off():
+def test_union_with_flag_conversion_off() -> None:
     """Union types should work with FlagConversionOff config."""
     result = tyro.cli(
         Union[UnionCommand1, UnionCommand2],
@@ -1982,7 +2016,7 @@ def test_union_with_flag_conversion_off():
     assert result == UnionCommand2(arg2=42, flag2=False)
 
 
-def test_union_with_omit_subcommand_prefixes():
+def test_union_with_omit_subcommand_prefixes() -> None:
     """Union types should work with OmitSubcommandPrefixes."""
     result = tyro.cli(
         Union[UnionCommand1, UnionCommand2],
@@ -1992,7 +2026,7 @@ def test_union_with_omit_subcommand_prefixes():
     assert result == UnionCommand1(arg1="test", flag1=False)
 
 
-def test_union_with_positional_required_args():
+def test_union_with_positional_required_args() -> None:
     """Union types should work with PositionalRequiredArgs."""
 
     @dataclasses.dataclass
@@ -2011,7 +2045,7 @@ def test_union_with_positional_required_args():
     assert result == SimpleUnionCmd1(required_arg="hello")
 
 
-def test_nested_union_with_config():
+def test_nested_union_with_config() -> None:
     """Nested union types should work with config."""
     result = tyro.cli(
         Union[UnionCommand1, UnionCommand3],
@@ -2021,7 +2055,7 @@ def test_nested_union_with_config():
     assert result == UnionCommand3(nested=NestedUnionConfig(value=2.5), name="test")
 
 
-def test_union_subcommand_help_with_config():
+def test_union_subcommand_help_with_config() -> None:
     """Test that help text works correctly with union types and config."""
     # This should not raise an exception
     with pytest.raises(SystemExit):
@@ -2032,7 +2066,7 @@ def test_union_subcommand_help_with_config():
         )
 
 
-def test_union_subcommand_specific_help_with_config():
+def test_union_subcommand_specific_help_with_config() -> None:
     """Test that subcommand-specific help works with config."""
     # This should not raise an exception
     with pytest.raises(SystemExit):
@@ -2041,3 +2075,38 @@ def test_union_subcommand_specific_help_with_config():
             args=["union-command1", "--help"],
             config=(tyro.conf.FlagCreatePairsOff,),
         )
+
+
+def test_conf_inheritance() -> None:
+    """Adapted from: https://github.com/brentyi/tyro/pull/328"""
+
+    @tyro.conf.configure(  # type: ignore
+        tyro.conf.arg(constructor_factory=lambda: Union[AdamConfig, SgdConfig])  # type: ignore
+    )
+    @dataclasses.dataclass
+    class OptimizerConfig:
+        lr: float
+
+    @tyro.conf.configure(
+        tyro.conf.subcommand(name="adam")  # type: ignore
+    )
+    @dataclasses.dataclass
+    class AdamConfig(OptimizerConfig):
+        betas: Tuple[float, float]
+
+    @dataclasses.dataclass
+    class SgdConfig(OptimizerConfig):
+        fused: bool
+
+    assert tyro.cli(
+        AdamConfig, args="--betas 0.99 0.95 --lr 1e-4".split(" ")
+    ) == AdamConfig(1e-4, (0.99, 0.95))
+    assert tyro.cli(SgdConfig, args="--lr 1e-4 --fused True".split(" ")) == SgdConfig(
+        1e-4, True
+    )
+    assert tyro.cli(
+        OptimizerConfig, args="adam --lr 1e-4 --betas 0.99 0.95".split(" ")
+    ) == AdamConfig(1e-4, (0.99, 0.95))
+    assert tyro.cli(
+        OptimizerConfig, args="sgd-config --lr 1e-4 --fused True".split(" ")
+    ) == SgdConfig(1e-4, fused=True)
